@@ -8,32 +8,13 @@ use File::Spec;
 use Env qw( @PATH );
 use FindBin ();
 use File::Temp qw( tempdir );
+use Cwd qw( cwd );
 
-sub ACTION_build
+sub _fetch_index1
 {
-  my $self = shift;
-
-  return $self->SUPER::ACTION_build(@_)
-    if $^O ne 'MSWin32' || do {
-      require lib;
-      lib->import('lib');
-      require Alien::MSYS;
-      # TODO: this means on a re-install we have to re-download
-      # if it has been a long time since the last version, this may
-      # not be a bad thing.  If we end up having lots of revisions
-      # this could be highly annoying.
-      do { no warnings 'redefine'; *Alien::MSYS::_my_dist_dir = sub {} };
-      defined Alien::MSYS::msys_path();
-    };
-
-  return $self->SUPER::ACTION_build(@_)
-    if $^O ne 'MSWin32' || $ENV{PERL_ALIEN_MSYS_BIN} || -d 'C:/MinGW/msys/1.0/bin';
-
-  require HTTP::Tiny;
-  my $http = HTTP::Tiny->new;
-  
+  require HTTP::Tiny;  
   my $url = 'http://sourceforge.net/projects/mingw/files/Installer/mingw-get/';
-  my $index = $http->get($url);
+  my $index = HTTP::Tiny->new->get($url);
   
   $index->{status} =~ /^2..$/ || die join(' ', $index->{status}, $index->{reason}, $url);
   
@@ -46,34 +27,82 @@ sub ACTION_build
       $link = {
         url  => "http://sourceforge.net/$1",
         date => $2,
-        num  => $2,
+        num  => $3,
       };
     }
   }
 
   die "couldn't find mingw-get in index" unless $link;
+  $link->{url};
+}
 
-  $url = $link->{url};
-  $index = $http->get($url);
+sub _fetch_index2
+{
+  my(undef, $url) = @_;
+  require HTTP::Tiny;  
+  my $index = HTTP::Tiny->new->get($url);
   
   $index->{status} =~ /^2..$/ || die join(' ', $index->{status}, $index->{reason}, $url);
 
   die "couldn't find mingw-get in download index"
     unless $index->{content} =~ m{"(https?://.*/(mingw-get-.*?-bin.zip)/download)"};
     
-  $url = $1;
-  my $zipname = $2;
-  print "url = $url\n";
-  print "zip = $zipname\n";
-  my $download = $http->get($url);
+  ($1, $2);
+}
 
+sub _fetch_zip
+{
+  my(undef, $url) = @_;
+  my $download = HTTP::Tiny->new->get($url);
   $download->{status} =~ /^2..$/ || die join(' ', $download->{status}, $download->{reason}, $url);
+  $download->{content};
+}
+
+sub ACTION_build
+{
+  my $self = shift;
+
+  my $override_type = $ENV{ALIEN_MSYS_INSTALL_TYPE} || $ENV{ALIEN_INSTALL_TYPE} || '';
+
+  return $self->SUPER::ACTION_build(@_)
+    if $^O ne 'MSWin32';
+
+  if($override_type ne 'share')
+  {
+    return $self->SUPER::ACTION_build(@_)
+      if do {
+        require lib;
+        lib->import('lib');
+        require Alien::MSYS;
+        # TODO: this means on a re-install we have to re-download
+        # if it has been a long time since the last version, this may
+        # not be a bad thing.  If we end up having lots of revisions
+        # this could be highly annoying.
+        do { no warnings 'redefine'; *Alien::MSYS::_my_dist_dir = sub {} };
+        defined Alien::MSYS::msys_path();
+      };
+
+    foreach my $try ($ENV{PERL_ALIEN_MSYS_BIN}, 'C:/MinGW/msys/1.0/bin')
+    {
+      my $sh_path = File::Spec->catfile($try, 'sh.exe');
+      return $self->SUPER::ACTION_build(@_) if -x $sh_path;
+    }
+  }
+
+  if($override_type eq 'system')
+  {
+    die "requested a system install, but could not be found!";
+  }
+
+  my($url, $zipname) = __PACKAGE__->_fetch_index2(__PACKAGE__->_fetch_index1);
+  my $zipcontent = __PACKAGE__->_fetch_zip($url);
 
   require Archive::Zip;
   
   my $dir = File::Spec->catdir($FindBin::Bin, qw( share ));
   mkpath($dir, 1, 0755);
   
+  my $save = cwd();
   chdir $dir;
   
   my $zip = Archive::Zip->new;
@@ -81,7 +110,7 @@ sub ACTION_build
     my $fn = File::Spec->catdir(tempdir(CLEANUP => 0), $zipname);
     open my $fh, '>', $fn;
     binmode $fh;
-    print $fh $download->{content};
+    print $fh $zipcontent;
     close $fh;
     print "fn = $fn\n";
     $fn;
@@ -97,17 +126,22 @@ sub ACTION_build
   }
   closedir $dh;
   
-  _cdup();
+  chdir $save;
 
   push @PATH, File::Spec->catdir($dir, qw( bin ));
   system 'mingw-get', 'install', 'msys';
+  
+  # A lot of tools also need m4
+  system 'mingw-get', 'install', 'msys-m4';
+  
+  # A number of autotools (autoconf, automake) are
+  # implemented in Perl, but when you build them
+  # with Perl on windows using MSYS, they use MSYS
+  # paths, like /c/ instead of c:/, so yes.  we
+  # also need MSYS Perl.  *sigh*
+  system 'mingw-get', 'install', 'msys-perl';
 
   $self->SUPER::ACTION_build(@_);
 }
 
-sub _cdup
-{
-  chdir(File::Spec->updir);
-}
-
 1;
