@@ -10,7 +10,7 @@ our @EXPORT    = qw( msys msys_run );
 our @EXPORT_OK = qw( msys msys_run msys_path );
 
 # ABSTRACT: Tools required for GNU style configure scripts on Windows
-our $VERSION = '0.08'; # VERSION
+our $VERSION = '0.09'; # VERSION
 
 
 sub msys (&)
@@ -68,9 +68,9 @@ sub msys_path ()
     }
 
     my $path = eval {
-      require File::HomeDir;
+      require Win32;
       require Win32::Shortcut;
-      my $lnk_name = File::Spec->catfile(File::HomeDir->my_desktop, 'MinGW Installer.lnk');
+      my $lnk_name = File::Spec->catfile(Win32::GetFolderPath(Win32::CSIDL_DESKTOP(), 1), 'MinGW Installer.lnk');
       die "No MinGW Installer.lnk" unless -r $lnk_name;
       my $lnk      = Win32::Shortcut->new;
       $lnk->Load($lnk_name);
@@ -114,7 +114,7 @@ Alien::MSYS - Tools required for GNU style configure scripts on Windows
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
